package ca.mcgill.sel.workflow.samples;

import ca.mcgill.sel.workflow.ConditionalExecutionNode;
import ca.mcgill.sel.workflow.ConditionalSynchronizationNode;
import ca.mcgill.sel.workflow.CustomizableInputData;
import ca.mcgill.sel.workflow.CustomizableNode;
import ca.mcgill.sel.workflow.CustomizableOutputData;
import ca.mcgill.sel.workflow.EndNode;
import ca.mcgill.sel.workflow.InputData;
import ca.mcgill.sel.workflow.InputNode;
import ca.mcgill.sel.workflow.LocalContext;
import ca.mcgill.sel.workflow.ParallelExecutionNode;
import ca.mcgill.sel.workflow.StartupNode;
import ca.mcgill.sel.workflow.SynchronizationNode;
import ca.mcgill.sel.workflow.WorkflowUtility;

public class AllNodesWorkflow extends AbstractWorkflowDemo {
    
    private class SomeInputData extends CustomizableInputData {

        private String data;
        
        public SomeInputData(String senderName, String queueName, String data) {
            super(senderName, queueName);
            
            this.data = data;
        }
        
        public String getData() {
            return data;
        }
        
    }
    
    private class ProcessInputNode extends CustomizableNode {
        
        @Override
        public void execute(LocalContext c) {
            InputData inputData = c.getMyInput();
            SomeInputData concreteInputData = (SomeInputData) inputData;
            System.out.println("Input received: " + concreteInputData.data);
        }
        
    }
    
    private class AskForInformationOutput extends CustomizableOutputData {
        
        private String message;
        
        public AskForInformationOutput(String message) {
            this.message = message;
        }
        
        public String getMessage() {
            return message;
        }
        
    }
    
    private class AskForInformation extends CustomizableNode {

        @Override
        public void execute(LocalContext c) {
            AskForInformationOutput output = new AskForInformationOutput("Gimme some information pls!");
            // TODO: simply output it on the console.
            System.out.println("Send over network: " + output.message);
            output.send("unknownHost");
        }
        
    }
    
    private class ChoosePathNode extends CustomizableNode {

        @Override
        public void execute(LocalContext c) {
            WorkflowUtility.getWorkflowUtility().setCurrentChosenPath("OrForkNodeName", "if");
        }
        
    }
    
    private class Node extends CustomizableNode {

        @Override
        public void execute(LocalContext c) {
            WorkflowUtility.getWorkflowUtility().setCurrentSyncCondition("CSNode", true);
        }
        
    }

    public AllNodesWorkflow(LocalContext context) {
        super(context);
    }

    @Override
    public void initNodes() {
        StartupNode startupNode = new StartupNode();
        EndNode endNode = new EndNode();
        
        ParallelExecutionNode parallelNode = new ParallelExecutionNode();
        ConditionalExecutionNode conditionalNode = new ConditionalExecutionNode("OrForkNodeName");
        conditionalNode.addNextNode("bla", endNode);
        ConditionalSynchronizationNode synchronizedNode2 = new ConditionalSynchronizationNode(false, "AndJoinNodeName");
        synchronizedNode2.setNextNode(null);
        conditionalNode.addNextNode(null, synchronizedNode2);
        AskForInformation requestNode = new AskForInformation();
        
        InputNode inputNode = new InputNode();
        ProcessInputNode processNode = new ProcessInputNode();
        
        
        HelloNode hello = new HelloNode("Hello from main path");
        HelloNode hello1 = new HelloNode("Hello from path 1");
        HelloNode hello2 = new HelloNode("Hello from path 2");
        HelloNode helloEnd = new HelloNode("Hello from end path");
        
        SynchronizationNode synchronizedNode = new SynchronizationNode();
        
        startupNode.setNextNode(hello);
        hello.setNextNode(parallelNode);
        
        parallelNode.addNextNode("path1", hello1);
        parallelNode.addNextNode("path2", hello2);
        
        hello1.setNextNode("waiting", synchronizedNode);
        hello2.setNextNode("trigger", synchronizedNode);
        
        synchronizedNode.setNextNode(helloEnd);
        helloEnd.setNextNode(endNode);
        
        workflow.addNode(startupNode);
        workflow.addNode(parallelNode);
        workflow.addNode(hello);
        workflow.addNode(hello1);
        workflow.addNode(hello2);
        workflow.addNode(helloEnd);
        workflow.addNode(synchronizedNode);
        workflow.addNode(endNode);
        
        workflow.addStartupNode(startupNode);
    }

    public static void main(String[] args) throws InterruptedException {
        LocalContext context = new LocalContext();
        AllNodesWorkflow workflow = new AllNodesWorkflow(context);
        workflow.executeWorkflow();
        
        Thread.sleep(2000);
        
        SomeInputData inputData = workflow.new SomeInputData("unsure", "unsure", "teh data");
        inputData.execute();
    }

}
