package ca.mcgill.sel.workflow.samples;

import ca.mcgill.sel.workflow.ConditionalSynchronizationNode;
import ca.mcgill.sel.workflow.CustomizableNode;
import ca.mcgill.sel.workflow.EndNode;
import ca.mcgill.sel.workflow.LocalContext;
import ca.mcgill.sel.workflow.ParallelExecutionNode;
import ca.mcgill.sel.workflow.StartupNode;
import ca.mcgill.sel.workflow.SynchronizationNode;
import ca.mcgill.sel.workflow.WorkflowUtility;

public class ParallelConditionalSynchronizationWorkflow extends AbstractWorkflowDemo {
    
    private class Node extends CustomizableNode {

        @Override
        public void execute(LocalContext c) {
            WorkflowUtility.getWorkflowUtility().setCurrentSyncCondition("CSNode", true);
        }
        
    }

    public ParallelConditionalSynchronizationWorkflow(LocalContext context) {
        super(context);
    }

    @Override
    public void initNodes() {
        StartupNode startupNode = new StartupNode();
        EndNode endNode = new EndNode();
        
        ParallelExecutionNode parallelNode = new ParallelExecutionNode();
        
        HelloNode hello = new HelloNode("Hello from main path");
        HelloNode hello1 = new HelloNode("Hello from path 1");
        HelloNode hello2 = new HelloNode("Hello from path 2");
        HelloNode helloEnd = new HelloNode("Hello from end path");
        
        ConditionalSynchronizationNode synchronizedNode = new ConditionalSynchronizationNode(false, "AndJoinNodeName");
        
        startupNode.setNextNode(hello);
        hello.setNextNode(parallelNode);
        
        parallelNode.addNextNode("path1", hello1);
        parallelNode.addNextNode("path2", hello2);
        
        hello1.setNextNode("waiting", synchronizedNode);
        hello2.setNextNode("trigger", synchronizedNode);
        
        synchronizedNode.setNextNode(helloEnd);
        helloEnd.setNextNode(endNode);
        
        workflow.addNode(startupNode);
        workflow.addNode(parallelNode);
        workflow.addNode(hello);
        workflow.addNode(hello1);
        workflow.addNode(hello2);
        workflow.addNode(helloEnd);
        workflow.addNode(synchronizedNode);
        workflow.addNode(endNode);
        
        workflow.addStartupNode(startupNode);
    }

    public static void main(String[] args) {
        LocalContext context = new LocalContext();
        ParallelConditionalSynchronizationWorkflow workflow = new ParallelConditionalSynchronizationWorkflow(context);
        workflow.executeWorkflow();
    }

}
